/// <reference types="node" />
/**
 * Request an SSL certificate for the given app name signed by the devcert root certificate
 * authority. If devcert has previously generated a certificate for that app name on this machine,
 * it will reuse that certificate.
 *
 * If this is the first time devcert is being run on this machine, it will generate and attempt to
 * install a root certificate authority.
 *
 * Returns a promise that resolves with { keyPath, certPath, key, cert }, where `key` and `cert` are
 * Buffers with the contents of `keyPath` and `certPath`, respectively.
 */
export default function devcert(appName: string, options?: {
    installCertutil?: boolean;
}): Promise<{
    keyPath: string;
    certPath: string;
    key: Buffer;
    cert: Buffer;
}>;
