"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _graphql = require("../graphql");

var _default = new _graphql.GraphQLScalarType({
  name: 'Date',

  serialize(value) {
    // Valid string values from server side:
    // 2016-02-02
    // 2016-02-02T00:13:22Z
    // 2016-02-02T00:13:22.000Z
    if (typeof value === 'string' && /^(\d{4})-(\d{2})-(\d{2})(T((\d{2}):(\d{2}):(\d{2}))(\.(\d{1,3}))?Z)?$/.test(value)) {
      return value;
    }

    if (typeof value === 'number' && Number.isFinite(value)) {
      return new Date(value).toJSON();
    }

    if (!(value instanceof Date)) {
      throw new TypeError('Field error: value is not an instance of Date');
    }

    if (Number.isNaN(value.getTime())) {
      throw new TypeError('Field error: value is an invalid Date');
    } // will be serialized to '2019-01-10T08:55:04.913Z'


    return value.toJSON();
  },

  parseValue(value) {
    const date = new Date(value);

    if (Number.isNaN(date.getTime())) {
      throw new TypeError('Field error: value is an invalid Date');
    }

    return date;
  },

  parseLiteral(ast) {
    if (ast.kind === _graphql.Kind.INT) {
      return new Date(parseInt(ast.value, 10));
    }

    if (ast.kind !== _graphql.Kind.STRING) {
      throw new _graphql.GraphQLError(`Query error: Can only parse string or integer to Date but got a: ${ast.kind}`, [ast]);
    }

    const result = new Date(ast.value);

    if (Number.isNaN(result.getTime())) {
      throw new _graphql.GraphQLError('Query error: Invalid date', [ast]);
    }

    return result;
  }

});

exports.default = _default;