/// <reference types="node" />
import objectSorter = require('./objectSorter');
import { HexBase64Latin1Encoding } from 'crypto';
declare namespace hasher {
    /**
     * Object hasher options
     */
    interface HasherOptions extends objectSorter.SorterOptions {
        /**
         * Hash algorithm to use
         * @default 'sha256'
         */
        alg?: string;
        /**
         * String encoding for hash
         * @default 'hex'
         */
        enc?: HexBase64Latin1Encoding;
    }
    interface Hasher {
        /**
         * Create hash of an object
         * @param object source object
         * @returns hash string of an object
         */
        hash(object: any, opts?: hasher.HasherOptions): string;
        /**
         * Create sorted string from an object
         * @param object source object
         * @returns sorted string from an object
         */
        sort(object: any): string;
        /**
         * Create sorted string from an object
         * @param object source object
         * @returns sorted string from an object
         * @alias sort
         */
        sortObject(object: any): string;
    }
}
/**
 * Hasher constructor
 * @param options hasher options
 */
declare function hasher(options?: hasher.HasherOptions): hasher.Hasher;
export = hasher;
//# sourceMappingURL=hasher.d.ts.map